package org.apache.bcel.verifier.structurals;

import org.apache.bcel.generic.CodeExceptionGen;
import java.util.HashSet;
import java.util.HashMap;
import org.apache.bcel.generic.MethodGen;
import java.util.Set;
import org.apache.bcel.generic.InstructionHandle;
import java.util.Map;

public class ExceptionHandlers
{
    private static final ExceptionHandler[] EMPTY_ARRAY;
    private final Map<InstructionHandle, Set<ExceptionHandler>> exceptionHandlers;
    
    public ExceptionHandlers(final MethodGen mg) {
        this.exceptionHandlers = new HashMap<InstructionHandle, Set<ExceptionHandler>>();
        final CodeExceptionGen[] exceptionHandlers;
        final CodeExceptionGen[] cegs = exceptionHandlers = mg.getExceptionHandlers();
        for (final CodeExceptionGen ceg : exceptionHandlers) {
            final ExceptionHandler eh = new ExceptionHandler(ceg.getCatchType(), ceg.getHandlerPC());
            for (InstructionHandle ih = ceg.getStartPC(); ih != ceg.getEndPC().getNext(); ih = ih.getNext()) {
                this.exceptionHandlers.computeIfAbsent(ih, k -> new HashSet()).add(eh);
                this.exceptionHandlers.computeIfAbsent(ih, k -> new HashSet()).add(eh);
            }
        }
    }
    
    public ExceptionHandler[] getExceptionHandlers(final InstructionHandle ih) {
        final Set<ExceptionHandler> hsSet = this.exceptionHandlers.get(ih);
        if (hsSet == null) {
            return ExceptionHandlers.EMPTY_ARRAY;
        }
        return hsSet.toArray(ExceptionHandler.EMPTY_ARRAY);
    }
    
    static {
        EMPTY_ARRAY = new ExceptionHandler[0];
    }
}
